"use string";

var p = require("packrattle");
var _ = require("underscore");

var whitespaceP = p.regex(/\s*/);

function lexemeP(parser) {
  return p.seq(parser, whitespaceP).onMatch(_.first);
}

function unquote(m) {
  return m[1].replace(/\\(.)/g, function (unused, c) {
    return c;
  });
}

function quote(str) {
  return str.replace(/(['\\])/g, function (unused, c) {
    return "\\" + c;
  });
}

var sexprP = p.alt(
  p.seq(lexemeP("("), p.repeat(function () { return sexprP; }), lexemeP(")")).onMatch(function (arr) { return arr[1]; }),
  lexemeP(p.regex(/[a-zA-Z\?\.\-\/*+<>=!%,~\$][a-zA-Z0-9_\?\.\-\/*+<>=+!%,~\$]*/)).onMatch(_.first),
  lexemeP(p.regex(/"((?:[^"]|\\.)*)"/)).onMatch(unquote),
  lexemeP(p.regex(/'((?:[^']|\\.)*)'/)).onMatch(unquote),
  lexemeP(p.regex(/[0-9]+/)).onMatch(function (m) { return parseInt(_.first(m), 10); } )
);

function parse(input) {
  var parser = p.seq(sexprP, p.end).onMatch(_.last);
  var res = p.consume(parser, input);

  if (res.ok) {
    return res.match;
  } else {
    throw new Error(res.message);
  }
}

function stringifyString(sexpr) {
  if (sexpr === "") {
    return "''";
  } else if (sexpr.match(/[ '"]/) || sexpr.match(/^\d*$/)) {
    return "'" + quote(sexpr) + "'";
  } else {
    return sexpr;
  }
}

function stringify(sexpr) {
  if (_.isString(sexpr)) {
    return stringifyString(sexpr);
  } else if (_.isNumber(sexpr)) {
    return ""+sexpr;
  } else if (_.isArray(sexpr)) {
    return "(" + sexpr.map(stringify).join(" ") + ")";
  }

  throw new Error("sexpr should be an array, a number or a string");
}

module.exports = {
  parse: parse,
  stringify: stringify,
};
